@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 # @Author: yafes
 # @Date:   2018-11-20 17:39:54
-# @Last Modified by:   Brady Endres
-# @Last Modified time: 2021-08-30
+# @Last Modified by:   Eric Palakovich Carr
+# @Last Modified time: 2023-03-21
 
 from __future__ import absolute_import
 
 from .syncsketch import SyncSketchAPI
 
-__version__ = "1.0.8.12"
+__version__ = "1.0.9.0"
 __author__ = "SyncSketch Dev Team"
-__credits__ = "Philip Floetotto, Yafes Sahin, Brady Endres"
+__credits__ = "Philip Floetotto, Yafes Sahin, Brady Endres, Eric Palakovich Carr"
